package com.jiayue.simulation.ice102;

import com.jiayue.simulation.ice102.service.ClientService;
import com.jiayue.simulation.nettyice102.NettyConsoleInterface;
import com.jiayue.simulation.util.SocketConstant;
import com.jiayue.simulation.util.SpringContextHolder;
import io.netty.buffer.ByteBufUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;

import java.io.IOException;
import java.net.InetSocketAddress;
import java.nio.ByteBuffer;
import java.nio.channels.SocketChannel;

@Slf4j
public class Client implements NettyConsoleInterface {

    //对端ip
    private String ip;

    //对端端口
    private Integer port;

    //通道状态
    private Boolean status = true;

    SocketChannel channel;

    @Autowired
    ClientService clientService = SpringContextHolder.getApplicationContext().getBean(ClientService.class);

    public Client(String ip, Integer port,Boolean status) {
        this.ip = ip;
        this.port = port;
        this.status = status;
        start();
    }

    /**
     * 客户端开启
     */
    @Override
    public void start() {
        //创建远程地址
        InetSocketAddress address = new InetSocketAddress(ip, port);

        //定义缓存
        ByteBuffer buffer = ByteBuffer.allocate(1024);
        try {
            //开启通道
            channel = SocketChannel.open();
            //连接远程远程服务器
            channel.connect(address);

            Thread.currentThread().setName("客戶端线程:"+ip+":"+port);

            String localAddress = channel.getLocalAddress().toString();
            SocketConstant.socketChannelMap.put(localAddress,channel);

            String info = "1040FFFF3E16";
            while (true) {
                if(SocketConstant.socketChannelMap.get(localAddress)!=null){
                    Thread.sleep(1000);
                    log.info("发送：{}",info);

                    channel.write(clientService.sendInfo(info));
//                    //控制台输入数据写到缓存
//                    buffer.put(info.getBytes("UTF-8"));
//                    //重置buffer 游标
//                    buffer.flip();
//                    //数据发送到数据
//                    channel.write(buffer);
//                    //清空缓存数据
//                    buffer.clear();
                    //读取服务器返回的数据
                    int readLen = channel.read(buffer);
                    if (readLen == -1) {
                        break;
                    }
                    //重置buffer游标
                    buffer.flip();
                    byte[] bytes = new byte[buffer.remaining()];
                    //读取数据到字节数组
                    buffer.get(bytes);

                    String msg = ByteBufUtil.hexDump(bytes).toUpperCase();
                    log.info("接收：{}",msg);
                    info = clientService.sendMessage(msg);
                    buffer.clear();
                }else {
                    channel.close();
                    log.info("{}通道关闭",localAddress);
                    this.status = false;
                    break;
                }

            }
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            if (null != channel) {
                try {
                    if(channel.isConnected()){
                        channel.close();
                    }
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
    }


    /**
     * 客户端关闭
     *
     * @param lcoalAdress
     */
    public void shutdown(String lcoalAdress){
        SocketConstant.socketChannelMap.remove(lcoalAdress);

    }



}
